# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces.ricevuta_pagamento import IRicevutaPagamento
from plone.dexterity.content import Container
from zope.interface import implementer


@implementer(IRicevutaPagamento)
class RicevutaPagamento(Container):
    """ """



from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone import api

from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_INTEGRATION_TESTING,
)

import unittest


class MoveNewsItemView(unittest.TestCase):

    layer = DESIGN_PLONE_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.view = api.content.get_view(
            "move_news_items", context=self.portal, request=self.request
        )

        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.news_container = api.content.create(
            type="Folder",
            title="News container",
            container=self.portal,
        )
        self.news_item = api.content.create(
            type="News Item",
            title="news item",
            tipologia_notizia="Notizia",
            container=self.portal,
        )
        self.news_item1 = api.content.create(
            type="News Item",
            title="news item1",
            tipologia_notizia="Notizia",
            container=self.news_container,
        )

    def test_news_result(self):
        self.view.request.form["news_type"] = "Notizia"

        result = self.view.news_results()

        self.assertIn(self.news_item.UID(), [i.UID for i in result])
        self.assertIn(self.news_item1.UID(), [i.UID for i in result])

    def test_move_data(self):
        self.view.request.form[self.news_item.UID()] = "on"
        self.view.request.form[self.news_item1.UID()] = "on"
        self.view.request.form["move"] = "true"
        self.view.request.form["to_path"] = "/".join(
            self.news_container.getPhysicalPath()
        )

        self.view.move_data()

        self.assertIn(self.news_item, self.news_container.listFolderContents())
        self.assertIn(self.news_item1, self.news_container.listFolderContents())

    def test_bad_data_endurance(self):
        """Test instatnce endurance to bad data"""
        bad_uid = "bad_uid"
        bad_path = "bad_path"
        self.view.request.form["move"] = "true"

        self.view.request.form["bad_uid"] = "on"

        self.view.move_data()

        self.view.request.form.pop(bad_uid)

        self.view.request.form[self.news_item.UID()] = "on"
        self.view.request.form["to_path"] = bad_path

        self.view.move_data()



# -*- coding: utf-8 -*-

"""Setup tests for this package."""
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession

import transaction
import unittest


class TestUpdateNoteBehavior(unittest.TestCase):

    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_is_enabled_in_bando(self):

        portal_types = api.portal.get_tool(name="portal_types")
        self.assertIn(
            "design.plone.contenttypes.behavior.update_note",
            portal_types["Bando"].behaviors,
        )

    def test_if_compiled_is_stored_in_metadata(self):
        bando = api.content.create(
            container=self.portal, type="Bando", title="Test Bando", update_note="Foo"
        )
        transaction.commit()
        resp = self.api_session.get(
            "/@search", params={"UID": bando.UID(), "metadata_fields": "update_note"}
        )

        res = resp.json()

        self.assertEqual(res["items"][0]["update_note"], bando.update_note)



# -*- coding: utf-8 -*-
from plone import api
from plone.restapi.interfaces import IExpandableElement
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.services import Service
from Products.CMFCore.interfaces import IFolderish
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter
from zope.globalrequest import getRequest
from zope.interface import implementer
from zope.interface import Interface


TRASPARENZA_FIELDS = [
    "modalita_avvio",
    "descrizione",
    "soggetti_esterni",
    "decorrenza_termine",
    "dove_rivolgersi",
    "dove_rivolgersi_extra",
    "fine_termine",
    "silenzio_assenso",
    "ufficio_responsabile",
    "provvedimento_finale",
    "organo_competente_provvedimento_finale",
    "modalita_richiesta_informazioni",
    "procedura_online",
    "altre_modalita_invio",
    "atti_documenti_corredo",
    "reperimento_modulistica",
    "pagamenti",
    "strumenti_tutela",
    "titolare_potere_sostitutivo",
    "customer_satisfaction",
    "riferimenti_normativi",
    "tempo_medio",
    "file_correlato",
    "responsabile_procedimento",
    "dirigente",
]


class TrasparenzaService(Service):
    def reply(self):
        catalog = api.portal.get_tool("portal_catalog")
        brains = catalog(**{"UID": self.request.uid})[0]
        obj = getMultiAdapter((brains.getObject(), getRequest()), ISerializeToJson)()
        result = {}
        for field in TRASPARENZA_FIELDS:
            result[field] = obj.get(field)
        return result


@implementer(IExpandableElement)
@adapter(Interface, Interface)
class TrasparenzaItems(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, expand=False):
        if "amministrazione-trasparente" not in self.context.absolute_url():
            return {}

        result = {
            "trasparenza-items": {
                "@id": "{}/@trasparenza-items".format(self.context.absolute_url())
            }
        }
        if not expand:
            return result
        data = self.get_trasparenza_data()
        if data:
            result["trasparenza-items"] = {"items": data}
        return result

    def get_trasparenza_data(self, context=None):
        if context is None:
            context = self.context
        res = []

        for child in context.listFolderContents():
            serializer = queryMultiAdapter(
                (child, self.request), ISerializeToJsonSummary
            )
            data = serializer()
            if child.portal_type == "Document":

                if IFolderish.providedBy(child):
                    children = [
                        x
                        for x in self.get_trasparenza_data(context=child)
                        if x.get("@type", "")
                        in [
                            "Document",
                        ]
                    ]
                    if children:
                        data["items"] = children
            res.append(data)
        return res


class TrasparenzaItemsGet(Service):
    def reply(self):
        data = TrasparenzaItems(self.context, self.request)
        return data(expand=True)["trasparenza-items"]



from collective.taxonomy.vocabulary import Vocabulary


def eea_api_taxonomy_taxonomy_call(self, context):

    if not self.data:
        return Vocabulary(self.name, {}, {}, {}, 2)

    request = getattr(context, "REQUEST", None)
    language = self.getCurrentLanguage(request)
    return self.makeVocabulary(language)



# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from collective.z3cform.datagridfield.datagridfield import DataGridFieldFactory
from collective.z3cform.datagridfield.row import DictRow
from design.plone.contenttypes import _
from design.plone.contenttypes.interfaces import IDesignPloneContentType
from plone.app.dexterity import textindexer
from plone.app.z3cform.widget import DateFieldWidget
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.namedfile import field
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema


class ITempiEScadenzeValueSchema(model.Schema):
    data_scadenza = schema.Date(
        title=_("data_scadenza_label", default="Data scadenza"),
        description=_(
            "data_scadenza_help",
            default="Data di scadenza della fase",
        ),
        required=False,
    )
    milestone = schema.TextLine(
        title=_("milestone_label", default="Titolo"),
        description=_(
            "milestone_help",
            default="Titolo della fase",
        ),
        required=True,
        default="",
    )
    interval_qt = schema.TextLine(
        title=_("interval_qt_label", default="Intervallo"),
        description=_(
            "interval_qt_help",
            default="Intervallo della fase",
        ),
        required=False,
        default="",
    )
    interval_type = schema.TextLine(
        title=_("interval_type_label", default="Tipo intervallo"),
        description=_(
            "interval_type_help",
            default="Ad esempio: " "ore, giorni, settimane, mesi.",
        ),
        required=False,
        default="",
    )
    milestone_description = schema.TextLine(
        title=_("milestone_description_label", default="Sottotitolo"),
        description=_(
            "milestone_description_help",
            default="Sottotitolo della fase",
        ),
        required=False,
        default="",
    )

    form.widget(
        "data_scadenza",
        DateFieldWidget,
    )


class IServizio(model.Schema, IDesignPloneContentType):
    """Marker interface for content type Servizio"""

    sottotitolo = schema.TextLine(
        title=_("sottotitolo_label", default="Sottotitolo"),
        description=_(
            "sottotitolo_help",
            default="Indica un eventuale sottotitolo/titolo alternativo.",
        ),
        required=False,
    )

    # TODO: stato servizio vuol dire si o no? Inoltre, deve essere visibile
    # solo se il servizio
    # non e' attivo
    stato_servizio = schema.Bool(
        title=_("stato_servizio_label", default="Servizio non fruibile"),
        required=False,
        description=_(
            "stato_servizio_help",
            default="Indica se il servizio è effettivamente fruibile; spuntare se non è fruibile.",  # noqa
        ),
    )

    motivo_stato_servizio = BlocksField(
        title=_(
            "motivo_stato_servizio_label",
            default="Motivo dello stato",
        ),
        required=False,
        description=_(
            "motivo_stato_servizio_help",
            default="Descrizione del motivo per cui il servizio non è attivo. È obbligatorio se il campo precedente è spuntato.",  # noqa
        ),
    )

    a_chi_si_rivolge = BlocksField(
        title=_("a_chi_si_rivolge_label", default="A chi è rivolto"),
        required=True,
        description=_(
            "a_chi_si_rivolge_help",
            default="A chi si rivolge questo servizio e chi può usufruirne.",
        ),
    )

    chi_puo_presentare = BlocksField(
        title=_("chi_puo_presentare_label", default="Chi può presentare"),
        required=False,
        description=_(
            "chi_puo_presentare_help",
            default="Descrizione di chi può presentare domanda per usufruire"
            " del servizio e delle diverse casistiche.",
        ),
    )

    copertura_geografica = BlocksField(
        title=_("copertura_geografica_label", default="Copertura geografica"),
        required=False,
        description=_(
            "copertura_geografica_help",
            default="Indicare se il servizio si riferisce ad una particolare"
            " area geografica o all'intero territorio di riferimento.",
        ),
    )

    come_si_fa = BlocksField(
        title=_("come_si_fa", default="Come fare"),
        required=True,
        description=_(
            "come_si_fa_help",
            default="Descrizione della procedura da seguire per poter"
            " usufruire del servizio.",
        ),
    )

    cosa_si_ottiene = BlocksField(
        title=_("cosa_si_ottiene", default="Cosa si ottiene"),
        description=_(
            "cosa_si_ottiene_help",
            default="Indicare cosa si può ottenere dal servizio, ad esempio"
            " 'carta di identità elettronica', 'certificato di residenza'.",
        ),
        required=True,
    )

    procedure_collegate = BlocksField(
        title=_("procedure_collegate", default="Procedure collegate all'esito"),
        required=False,
        description=_(
            "procedure_collegate_help",
            default="Indicare cosa deve fare l'utente del servizio per"
            " conoscere l'esito della procedura, e dove eventualmente"
            " poter ritirare l'esito.",
        ),
    )

    canale_digitale = BlocksField(
        title=_("canale_digitale", default="Canale digitale"),
        description=_(
            "canale_digitale_help",
            default="Testo di introduzione del canale digitale",
        ),
        required=False,
    )

    canale_digitale_link = schema.URI(
        title=_("canale_digitale_link", default="Link al canale digitale"),
        description=_(
            "canale_digitale_link_help",
            default="Collegamento con l'eventuale canale digitale di"
            " attivazione del servizio.",
        ),
        required=False,
    )
    # vocabolario dalle unita' organizzative presenti a catalogo?
    canale_fisico = RelationList(
        title=_("canale_fisico", default="Canale fisico"),
        description=_(
            "canale_fisico_help",
            default="Unità organizzative per la fruizione del servizio",
        ),
        required=True,
        default=[],
        value_type=RelationChoice(
            title=_("Canale fisico"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
    )

    autenticazione = BlocksField(
        title=_("autenticazione", default="Autenticazione"),
        description=_(
            "autenticazione_help",
            default="Indicare, se previste, le modalità di autenticazione"
            " necessarie per poter accedere al servizio.",
        ),
        required=False,
    )

    dove_rivolgersi = RelationList(
        title="Dove rivolgersi",
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
        description=_(
            "dove_rivolgersi_help",
            default="Seleziona una lista delle sedi e dei luoghi in cui è presente"
            " questo servizio.",
        ),
    )

    dove_rivolgersi_extra = BlocksField(
        title=_(
            "dove_rivolgersi_extra",
            default="Dove rivolgersi: informazioni aggiuntive",
        ),
        description=_(
            "dove_rivolgersi_extra_help",
            default="Indicare eventuali informazioni aggiuntive riguardo al dove "
            "rivolgersi per questo servizio.",
        ),
        required=False,
    )

    prenota_appuntamento = BlocksField(
        title=_("prenota_appuntamento", default="Prenota un appuntamento"),
        description=_(
            "prenota_appuntamento_help",
            default="Se è possibile prenotare un'appuntamento, indicare"
            " le informazioni necessarie e il collegamento al servizio di "
            "prenotazione appuntamenti del Comune.",
        ),
        required=False,
    )

    tempi_e_scadenze = BlocksField(
        title=_("tempi_e_scadenze", default="Tempi e scadenze"),
        required=True,
        description=_(
            "tempi_e_scadenze_help",
            default="Descrivere le informazioni dettagliate riguardo eventuali tempi"
            " e scadenze di questo servizio.",
        ),
    )

    timeline_tempi_scadenze = schema.List(
        title=_("timeline_tempi_scadenze", default="Timeline tempi e scadenze"),
        default=[],
        value_type=DictRow(schema=ITempiEScadenzeValueSchema),
        description=_(
            "timeline_tempi_scadenze_help",
            default="Timeline tempi e scadenze del servizio: indicare per ogni "
            "scadenza un titolo descritttivo di tale scadenza e, opzionalmente,"
            " informazioni sulle date o gli intervalli di tempo che "
            "intercorrono tra una fase e la successiva.",
        ),
        required=False,
    )

    cosa_serve = BlocksField(
        title=_("cosa_serve", default="Cosa serve"),
        required=True,
        description=_(
            "cosa_serve_help",
            default="Descrizione delle istruzioni per usufruire del servizio.",
        ),
    )

    costi = BlocksField(
        title=_("costi", default="Costi"),
        required=False,
        description=_(
            "costi_help",
            default="Descrizione delle condizioni e dei termini economici per"
            " completare la procedura di richiesta del servizio.",
        ),
    )

    vincoli = BlocksField(
        title=_("vincoli", default="Vincoli"),
        required=False,
        description=_(
            "vincoli_help",
            default="Descrizione degli eventuali vincoli presenti.",
        ),
    )

    casi_particolari = BlocksField(
        title=_("casi_particolari", default="Casi particolari"),
        required=False,
        description=_(
            "casi_particolari_help",
            default="Descrizione degli evetuali casi particolari riferiti"
            " alla fruibilità di questo servizio.",
        ),
    )

    # vocabolario dalle unita' organizzative presenti a catalogo?
    ufficio_responsabile = RelationList(
        title=_(
            "ufficio_responsabile_erogazione",
            default="Unità organizzativa responsabile",
        ),
        description=_(
            "ufficio_responsabile_help",
            default="Seleziona gli uffici responsabili dell'erogazione"
            " di questo servizio.",
        ),
        required=True,
        default=[],
        value_type=RelationChoice(
            title=_("Unità organizzativa responsabile"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
    )

    area = RelationList(
        title=_("area", default="Area"),
        required=False,
        default=[],
        description=_(
            "area_help",
            default="Seleziona l'area da cui dipende questo servizio.",
        ),
        value_type=RelationChoice(
            title=_("Area"), vocabulary="plone.app.vocabularies.Catalog"
        ),
    )

    altri_documenti = RelationList(
        title="Documenti",
        default=[],
        description=_(
            "altri_documenti_help",
            default="Seleziona la lista dei documenti di supporto collegati"
            " a questo servizio.",
        ),
        value_type=RelationChoice(
            title=_("Documento"), vocabulary="plone.app.vocabularies.Catalog"
        ),
        required=False,
    )

    link_siti_esterni = BlocksField(
        title=_("link_siti_esterni", default="Link a siti esterni"),
        required=False,
        description=_(
            "link_siti_esterni_help",
            default="Eventuali collegamenti a pagine web, siti, servizi"
            " esterni all'ambito Comunale utili all'erogazione del servizio.",
        ),
    )

    codice_ipa = schema.TextLine(
        title=_("codice_ipa", default="Codice dell'ente erogatore (ipa)"),
        required=False,
        description=_(
            "codice_ipa_help",
            default="Specificare il nome dell’organizzazione, come indicato"
            " nell’Indice della Pubblica Amministrazione (IPA), che esercita"
            " uno specifico ruolo sul Servizio.",
        ),
    )

    # classificazione basata sul catalogo dei servizi, stringa o lista?
    settore_merceologico = schema.TextLine(
        title=_("settore_merceologico", default="Settore merceologico"),
        required=False,
        description=_(
            "settore_merceologico_help",
            default="Classificazione del servizio basata su catalogo dei"
            " servizi (Classificazione NACE).",
        ),
    )

    identificativo = schema.TextLine(
        title=_("identificativo", default="Identificativo"),
        required=False,
        description=_(
            "identificativo_help",
            default="Eventuale codice identificativo del servizio.",
        ),
    )

    condizioni_di_servizio = field.NamedBlobFile(
        title=_("condizioni_di_servizio", default="Condizioni di servizio"),
        required=True,
    )

    servizi_collegati = RelationList(
        title=_("servizi_collegati_label", default="Servizi collegati"),
        description=_(
            "servizi_collegati_help",
            default="Seleziona la lista dei servizi collegati a questo.",
        ),
        default=[],
        value_type=RelationChoice(
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        required=False,
    )

    # custom widgets
    form.widget(
        "canale_fisico",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 10,
            "selectableTypes": ["UnitaOrganizzativa"],
        },
    )
    form.widget(
        "dove_rivolgersi",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 10,
            "selectableTypes": ["Venue", "UnitaOrganizzativa"],
        },
    )

    form.widget(
        "area",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 1,
            "selectableTypes": ["UnitaOrganizzativa"],
            # "basePath": "/amministrazione/aree-amministrative",
        },
    )
    form.widget(
        "ufficio_responsabile",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["UnitaOrganizzativa"],
            # "basePath": "/amministrazione/uffici",
        },
    )
    form.widget(
        "altri_documenti",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 10,
            "selectableTypes": ["Documento", "CartellaModulistica"],
            # "basePath": "/",
        },
    )
    form.widget(
        "servizi_collegati",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 10,
            "selectableTypes": ["Servizio"],
            # "basePath": "/",
        },
    )
    form.widget(
        "timeline_tempi_scadenze",
        DataGridFieldFactory,
        frontendOptions={"widget": "data_grid"},
    )

    # custom fieldset and order
    model.fieldset(
        "a_chi_si_rivolge",
        label=_("a_chi_si_rivolge_label", default="A chi si rivolge"),
        fields=["a_chi_si_rivolge", "chi_puo_presentare", "copertura_geografica"],
    )

    model.fieldset(
        "accedi_al_servizio",
        label=_("accedi_al_servizio_label", default="Accedere al servizio"),
        fields=[
            "come_si_fa",
            "cosa_si_ottiene",
            "procedure_collegate",
            "canale_digitale",
            "canale_digitale_link",
            "canale_fisico",
            "autenticazione",
            "dove_rivolgersi",
            "dove_rivolgersi_extra",
            "prenota_appuntamento",
        ],
    )
    model.fieldset(
        "cosa_serve",
        label=_("cosa_serve_label", default="Cosa serve"),
        fields=["cosa_serve"],
    )
    model.fieldset(
        "costi_e_vincoli",
        label=_("costi_e_vincoli_label", default="Costi e vincoli"),
        fields=["costi", "vincoli"],
    )

    model.fieldset(
        "tempi_e_scadenze",
        label=_("tempi_e_scadenze_label", default="Tempi e scadenze"),
        fields=["tempi_e_scadenze", "timeline_tempi_scadenze"],
    )

    model.fieldset(
        "casi_particolari",
        label=_("casi_particolari_label", default="Casi particolari"),
        fields=["casi_particolari"],
    )

    model.fieldset(
        "contatti",
        label=_("contatti_label", default="Contatti"),
        fields=["ufficio_responsabile", "area"],
    )
    model.fieldset(
        "documenti",
        label=_("documenti_label", default="Documenti"),
        fields=["altri_documenti"],
    )
    model.fieldset(
        "link_utili",
        label=_("link_utili_label", default="Link utili"),
        fields=["link_siti_esterni"],
    )

    model.fieldset(
        "correlati",
        label=_("correlati_label", default="Contenuti collegati"),
        fields=["servizi_collegati"],
    )

    model.fieldset(
        "categorization",
        fields=["codice_ipa", "settore_merceologico", "identificativo"],
    )

    # SearchableText fields
    textindexer.searchable("sottotitolo")
    textindexer.searchable("a_chi_si_rivolge")
    textindexer.searchable("chi_puo_presentare")
    textindexer.searchable("come_si_fa")
    textindexer.searchable("cosa_si_ottiene")
    textindexer.searchable("cosa_serve")
    textindexer.searchable("area")
    textindexer.searchable("ufficio_responsabile")
    textindexer.searchable("copertura_geografica")
    textindexer.searchable("costi")
    textindexer.searchable("servizi_collegati")
    textindexer.searchable("link_siti_esterni")



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces.punto_di_contatto import IPuntoDiContatto
from plone.app.dexterity.textindexer.interfaces import IDynamicTextIndexExtender
from zope.component import adapter
from zope.interface import implementer


@implementer(IDynamicTextIndexExtender)
@adapter(IPuntoDiContatto)
class PuntoDiContattoMoreTextToIndex(object):
    def __init__(self, context):
        self.context = context

    def __call__(self):
        """Extend the searchable text with a custom string"""
        result = []
        field_value = getattr(self.context, "value_punto_contatto", [])
        for value in field_value:
            result.append(value.get("pdc_value", ""))
        return " ".join(result)



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces.bando import IBandoAgidSchema
from plone.indexer.decorator import indexer


@indexer(IBandoAgidSchema)
def ufficio_responsabile_bando(context, **kw):
    uffici = getattr(context.aq_base, "ufficio_responsabile", [])
    return [ufficio.UID() for ufficio in filter(bool, [x.to_object for x in uffici])]


@indexer(IBandoAgidSchema)
def Subject_bando(context, **kw):

    return context.Subject



# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from design.plone.contenttypes import _
from design.plone.contenttypes.interfaces import IDesignPloneContentType
from plone.app.dexterity import textindexer
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList


# TODO: migration script for these commented fields towards PDC
# contact_info
# Probabilmente non possibile trattandosi di un campo a blocchi
# preferirei si arrangiassero le redazioni. Altrimenti si defaulta
# ad un tipo a caso + tutto il testo e poi si arrangiano comunque
class IUnitaOrganizzativa(model.Schema, IDesignPloneContentType):
    """Marker interface for content type UnitaOrganizzativa"""

    competenze = BlocksField(
        title=_("uo_competenze_label", default="Competenze"),
        description=_(
            "uo_competenze_help",
            default="Descrizione dei compiti assegnati alla struttura.",
        ),
        required=True,
    )

    legami_con_altre_strutture = RelationList(
        title=_(
            "legami_altre_strutture_label", default="Strutture o uffici di riferimento"
        ),
        default=[],
        description=_(
            "legami_con_altre_strutture_help",
            default="Selezionare la lista di strutture e/o uffici collegati"
            " a questa unità organizzativa.",
        ),
        value_type=RelationChoice(
            title=_("Struttura"), vocabulary="plone.app.vocabularies.Catalog"
        ),
        required=False,
    )

    responsabile = RelationList(
        title=_("responsabile_label", default="Responsabile"),
        value_type=RelationChoice(
            title=_("Responsabile"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        description=_(
            "responsabile_help",
            default="Selezionare il/i responsabile/i della struttura.",
        ),
        default=[],
        required=False,
    )

    assessore_riferimento = RelationList(
        title="Assessore di riferimento",
        # vocabolario di riferimento sara' dinamico con i content type
        # persona presenti all'interno della macro Amministrazione"
        value_type=RelationChoice(
            title=_("Assessore di riferimento"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        description=_(
            "assessore_riferimento_help",
            default="Inserire l'assessore di riferimento della struttura,"
            " se esiste.",
        ),
        required=False,
        default=[],
    )

    # vocabolario di riferimento sara' dinamico con i content type persona
    persone_struttura = RelationList(
        title=_(
            "persone_struttura_label", default="Persone che compongono la struttura"
        ),
        default=[],
        value_type=RelationChoice(
            title=_("Persone della struttura"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        description=_(
            "persone_struttura_help",
            default="Seleziona la lista delle persone che compongono" " la struttura.",
        ),
        required=True,
    )

    sede = RelationList(
        title=_("sede_label", default="Sede principale"),
        default=[],
        description=_(
            "sede_help",
            default="Seleziona il Luogo in cui questa struttura ha sede. "
            "Se non è presente un contenuto di tipo Luogo a cui far "
            "riferimento, puoi compilare i campi seguenti. Se selezioni un "
            "Luogo, puoi usare comunque i campi seguenti per sovrascrivere "
            "alcune informazioni.",
        ),
        value_type=RelationChoice(
            title=_("Sede"), vocabulary="plone.app.vocabularies.Catalog"
        ),
        required=True,
    )

    sedi_secondarie = RelationList(
        title=_("sedi_secondarie_label", default="Altre sedi"),
        default=[],
        description=_(
            "sedi_secondarie_help",
            default="Seleziona una lista di eventuali contenuti di tipo Luogo"
            " che sono sedi secondarie di questa struttura. "
            "Per queste sedi non sarà possibile sovrascrivere i dati. "
            "Nel caso servano informazioni diverse, è possibile usare il campo"
            " sottostante.",
        ),
        value_type=RelationChoice(
            title=_("Sede"), vocabulary="plone.app.vocabularies.Catalog"
        ),
        required=False,
    )

    documenti_pubblici = RelationList(
        title=_("documenti_pubblici_label", default="Documenti pubblici"),
        default=[],
        description=_(
            "documenti_pubblici_help",
            default="Documenti pubblici importanti, collegati a questa Unità Organizzativa",  # noqa
        ),
        value_type=RelationChoice(
            title=_("Documenti pubblici"), vocabulary="plone.app.vocabularies.Catalog"
        ),
        required=False,
    )

    #  custom widgets
    form.widget(
        "documenti_pubblici",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 10,
            "selectableTypes": ["Documento"],
        },
    )
    form.widget(
        "persone_struttura",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={"selectableTypes": ["Persona"], "maximumSelectionSize": 50},
    )
    form.widget(
        "legami_con_altre_strutture",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 10,
            "selectableTypes": ["UnitaOrganizzativa"],
        },
    )
    form.widget(
        "responsabile",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 1,
            "selectableTypes": ["Persona"],
            # "basePath": "/amministrazione",
        },
    )
    form.widget(
        "assessore_riferimento",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 1,
            "selectableTypes": ["Persona"],
            # "basePath": "/amministrazione",
        },
    )
    form.widget(
        "sede",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={"maximumSelectionSize": 1, "selectableTypes": ["Venue"]},
    )
    form.widget(
        "sedi_secondarie",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 10,
            "selectableTypes": ["Venue"],
            # "basePath": "/servizi",
        },
    )

    # custom fieldsets and order
    model.fieldset(
        "cosa_fa",
        label=_("cosa_fa_label", default="Competenze"),
        fields=["competenze"],
    )
    model.fieldset(
        "struttura",
        label=_("struttura_label", default="Struttura"),
        fields=[
            "legami_con_altre_strutture",
            "responsabile",
            "assessore_riferimento",
        ],
    )
    model.fieldset(
        "persone",
        label=_("persone_label", default="Persone"),
        fields=["persone_struttura"],
    )
    model.fieldset(
        "contatti",
        label=_("contatti_label", default="Contatti"),
        fields=["sede", "sedi_secondarie"],
    )

    model.fieldset(
        "correlati",
        label=_("correlati_label", default="Contenuti collegati"),
        fields=["documenti_pubblici"],
    )

    form.order_after(sedi_secondarie="IContattiUnitaOrganizzativa.orario_pubblico")
    form.order_after(documenti_pubblici="relatedItems")
    # form.order_after(contact_info="sedi_secondarie")

    # SearchableText indexers
    textindexer.searchable("competenze")
    textindexer.searchable("assessore_riferimento")
    textindexer.searchable("responsabile")



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces.pagina_argomento import IPaginaArgomento
from plone.app.dexterity.textindexer.interfaces import IDynamicTextIndexExtender
from plone.restapi.indexers import SearchableText_blocks
from zope.component import adapter
from zope.interface import implementer


@adapter(IPaginaArgomento)
@implementer(IDynamicTextIndexExtender)
class SearchableTextExtender(object):
    def __init__(self, context):
        self.context = context

    def __call__(self):
        """Extend the searchable text with blocks"""
        return SearchableText_blocks(self.context)()



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces.modulo import IModulo
from plone.dexterity.content import Item
from zope.interface import implementer


@implementer(IModulo)
class Modulo(Item):
    """Modulo"""



# -*- coding: utf-8 -*-
from Products.CMFPlone.interfaces import ISelectableConstrainTypes
from Products.CMFPlone.utils import _createObjectByType


def luogoCreateHandler(luogo, event):
    """
    Complete content type luogo setup on added event, generating
    missing folders, fields, etc.

    @param luogo: Content item

    @param event: Event that triggers the method (onAdded event)
    """
    folder_id = "multimedia"
    if folder_id in luogo:
        return
    folder = _createObjectByType("Folder", luogo, "multimedia")
    folder.title = "Multimedia"
    folder.reindexObject(idxs=["Title"])
    constraints = ISelectableConstrainTypes(folder)
    constraints.setConstrainTypesMode(1)
    constraints.setLocallyAllowedTypes(
        (
            "Image",
            "Link",
        )
    )

    # non dovrebbe essere cancellabile



# -*- coding: utf-8 -*-
from .related_news_serializer import SerializeFolderToJson as RelatedNewsSerializer
from Acquisition import aq_inner
from design.plone.contenttypes.interfaces.unita_organizzativa import IUnitaOrganizzativa
from design.plone.contenttypes.restapi.serializers.summary import (
    DefaultJSONSummarySerializer,
)
from plone import api
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.serializer.converters import json_compatible
from zc.relation.interfaces import ICatalog
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.globalrequest import getRequest
from zope.interface import implementer
from zope.interface import Interface
from zope.intid.interfaces import IIntIds
from zope.security import checkPermission


@implementer(ISerializeToJson)
@adapter(IUnitaOrganizzativa, Interface)
class UOSerializer(RelatedNewsSerializer):
    def get_services(self):
        """ """
        catalog = getUtility(ICatalog)
        intids = getUtility(IIntIds)
        services = []
        for attr in ["ufficio_responsabile", "area"]:
            relations = catalog.findRelations(
                dict(
                    to_id=intids.getId(aq_inner(self.context)),
                    from_attribute=attr,
                )
            )

            for rel in relations:
                obj = intids.queryObject(rel.from_id)
                if (
                    obj is not None
                    and checkPermission("zope2.View", obj)  # noqa
                    and obj.portal_type == "Servizio"  # noqa
                ):
                    summary = getMultiAdapter(
                        (obj, getRequest()), ISerializeToJsonSummary
                    )()
                    services.append(summary)
        return sorted(services, key=lambda k: k["title"])

    def getChildrenUo(self):
        res = []
        children = self.context.listFolderContents(
            contentFilter={"portal_type": "UnitaOrganizzativa"}
        )
        if not children:
            return []

        for child in children:
            data = getMultiAdapter((child, self.request), ISerializeToJsonSummary)()
            data.update(self.getAdditionalInfos(context=child))
            res.append(json_compatible(data))
        return res

    def getParentUo(self):
        parent = self.context.aq_parent
        if parent.portal_type != "UnitaOrganizzativa":
            return None

        data = getMultiAdapter((parent, self.request), ISerializeToJsonSummary)()
        data.update(self.getAdditionalInfos(context=parent))
        return json_compatible(data)

    def getAdditionalInfos(self, context):
        return {
            "city": getattr(context, "city", ""),
            "zip_code": getattr(context, "zip_code", ""),
            "street": getattr(context, "street", ""),
            "contact_info": getattr(context, "contact_info", ""),
        }

    def getUOServiziDoveRivolgersi(self, UID):
        """Returns list of servizio having reference to
        current object in servizio.dove_rivolgersi field
        """
        catalog = api.portal.get_tool("portal_catalog")
        query = {
            "service_venue": UID,
            "portal_type": ["Servizio"],
            "sort_on": "sortable_title",
            "sort_order": "ascending",
        }

        return [
            getMultiAdapter((i, self.request), ISerializeToJsonSummary)()
            for i in catalog(**query)
        ]

    def __call__(self, version=None, include_items=True):
        self.index = "news_uo"
        result = super(UOSerializer, self).__call__(
            version=version, include_items=include_items
        )
        result["servizi_offerti"] = json_compatible(self.get_services())
        result["uo_parent"] = json_compatible(self.getParentUo())
        result["uo_children"] = json_compatible(self.getChildrenUo())
        result["prestazioni"] = json_compatible(
            self.getUOServiziDoveRivolgersi(result.get("UID", ""))
        )

        return result


@implementer(ISerializeToJsonSummary)
@adapter(IUnitaOrganizzativa, Interface)
class UOJSONSummarySerializer(DefaultJSONSummarySerializer):
    def __call__(self, force_all_metadata=False):
        data = super().__call__(force_all_metadata=force_all_metadata)
        fields = [
            "address",
            "city",
            "zip_code",
            # "email",
            # "telefono",
            "nome_sede",
            "title",
            "quartiere",
            "circoscrizione",
            "street",
            "contact_info",
        ]

        for field in fields:
            if field == "contact_info":
                data[field] = json_compatible(getattr(self.context, field, ""))
            else:
                data[field] = getattr(self.context, field, "")

        data["geolocation"] = self.getGeolocation()

        return data

    def getGeolocation(self):
        longitude = 0
        latitude = 0

        if getattr(self.context, "geolocation", None):
            longitude = getattr(self.context.geolocation, "longitude", 0)
            latitude = getattr(self.context.geolocation, "latitude", 0)

        return {"longitude": longitude, "latitude": latitude}



# -*- coding: utf-8 -*-
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.formwidget.geolocation import Geolocation
from plone.restapi.interfaces import ISerializeToJsonSummary
from zope.component import getMultiAdapter

import unittest


class UOSummarySerializerTest(unittest.TestCase):

    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]

    def test_geolocation_injected(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        uo = api.content.create(
            container=self.portal,
            type="UnitaOrganizzativa",
            title="UO",
            geolocation=Geolocation(25, 25),
        )

        summary = getMultiAdapter(
            (uo, self.layer["request"]), ISerializeToJsonSummary
        )()

        self.assertDictEqual(
            {
                "longitude": uo.geolocation.longitude,
                "latitude": uo.geolocation.latitude,
            },
            summary["geolocation"],
        )



from design.plone.contenttypes import _
from plone import api
from plone.z3cform.layout import wrap_form
from z3c.form import button
from z3c.form import field
from z3c.form import form
from zope import schema
from zope.component import getMultiAdapter
from zope.interface import implementer
from zope.interface import Interface


class ISearchForm(Interface):
    """
    Form to search Servizio by path
    """

    path = schema.TextLine(
        title=_(
            "licenza_distribuzione_label",
            default="Seleziona il percorso sotto il quale cercare",
        ),
        required=False,
    )


@implementer(ISearchForm)
class CheckServizi(form.Form):

    ignoreContext = True
    # template = ViewPageTemplateFile("templates/check_servizi.pt")
    prefix = ""
    brains = []
    fields = field.Fields(ISearchForm)
    method = "GET"

    def search_servizi(self):
        pc = api.portal.get_tool("portal_catalog")
        query = {"portal_type": "Servizio", "sort_on": "sortable_title"}
        if self.request.form.get("widgets.path") and self.request.form.get(
            "buttons.action_search"
        ):
            path = self.request.form.get("widgets.path")
            prefix = api.portal.get().getId()
            if not path.startswith(f"/{prefix}"):
                path = f"/{prefix}/{path}"
            query["path"] = {"query": path}
        brains = pc(**query)
        servizi = [brain.getObject() for brain in brains]
        results = []
        for servizio in servizi:
            results.append(
                {
                    "title": servizio.title,
                    "url": "/".join(servizio.getPhysicalPath()),
                    "breadcrumbs": getMultiAdapter(
                        (servizio, self.request), name="breadcrumbs_view"
                    ).breadcrumbs(),
                }
            )
        return results

    def updateWidgets(self):
        super(CheckServizi, self).updateWidgets()
        for k, v in self.request.form.items():
            if k in self.widgets:
                self.widgets[k].value = v

    @button.buttonAndHandler(_("action_search", default="Cerca"))
    def action_search(self, action):
        """
        Search in prenotazioni SearchableText
        """
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

    @button.buttonAndHandler(_("move_back_message", default="Reset"))
    def action_cancel(self, action):
        """
        Cancel and go back to the week view
        """
        target = self.context.absolute_url() + "/check_servizi"
        return self.request.response.redirect(target)


WrappedCheckServiziForm = wrap_form(CheckServizi)



from Acquisition import aq_base
from collective.taxonomy.interfaces import ITaxonomy
from copy import deepcopy
from design.plone.contenttypes import _
from logging import getLogger
from plone import api
from Products.Five.browser import BrowserView
from zope.component import getUtility
from zope.interface.interfaces import ComponentLookupError


logger = getLogger(__name__)


class View(BrowserView):
    """This view is needed to change the news type on the existent content"""

    def __call__(self, *args, **kwargs):
        self.substitute_news_type()
        return super().__call__(*args, **kwargs)

    def news_types(self):
        try:
            taxonomy = getUtility(
                ITaxonomy, name="collective.taxonomy.tipologia_notizia"
            )
        except ComponentLookupError:
            self.context.plone_utils.addPortalMessage(
                _("Il vocabolario dei valori non è stato trovato"), "error"
            )
            return

        return taxonomy.makeVocabulary(self.request.get("LANGUAGE"))

    def news_types_in_catalog(self):
        return api.portal.get_tool("portal_catalog").uniqueValuesFor(
            "tipologia_notizia"
        )

    def substitute_news_type(self):
        if not self.request.form.get("substitute", ""):
            return

        old_news_type = self.request.form.get("news_type_in_catalog", "")
        news_new_type = self.request.form.get("news_type_portal", "")

        if not old_news_type:
            self.context.plone_utils.addPortalMessage(
                _("The old type field was not populated"), "error"
            )
            return

        if not news_new_type:
            self.context.plone_utils.addPortalMessage(
                _("The new type field was not populated"), "error"
            )
            return

        if news_new_type not in self.news_types():
            self.context.plone_utils.addPortalMessage(
                _("The new News Type was not found between available values"), "error"
            )
            return

        if old_news_type not in self.news_types_in_catalog():
            self.context.plone_utils.addPortalMessage(
                _("The old News Type was not found between available values"), "error"
            )
            return

        for news in api.portal.get_tool("portal_catalog")(
            tipologia_notizia=old_news_type
        ):
            news = news.getObject()
            news.tipologia_notizia = news_new_type
            news.reindexObject(idxs=["tipologia_notizia"])

        # update listings
        for brain in api.portal.get_tool("portal_catalog")():
            item = aq_base(brain.getObject())

            if getattr(item, "blocks", {}):
                blocks = deepcopy(item.blocks)

                if blocks:
                    for block in blocks.values():
                        if block.get("@type", "") == "listing":
                            for query in block.get("querystring", {}).get("query", []):
                                if query["i"] == "tipologia_notizia":
                                    new_values = []
                                    for v in query["v"]:
                                        if v == old_news_type:
                                            v = news_new_type
                                        new_values.append(v)

                                    query["v"] = new_values

                                    logger.info(f"Updated listing {block}")

                    item.blocks = blocks

        self.context.plone_utils.addPortalMessage(
            _("The News Types was changed with success"), "info"
        )



# -*- coding: utf-8 -*-
from collective.z3cform.datagridfield.interfaces import IRow
from design.plone.contenttypes import _
from plone.restapi.types.adapters import ObjectJsonSchemaProvider
from plone.restapi.types.interfaces import IJsonSchemaProvider
from plone.restapi.types.utils import get_fieldsets
from plone.restapi.types.utils import get_jsonschema_properties
from plone.restapi.types.utils import iter_fields
from zope.component import adapter
from zope.component import getUtility
from zope.i18n import translate
from zope.interface import implementer
from zope.interface import Interface
from zope.schema.interfaces import IField
from zope.schema.interfaces import IVocabularyFactory


DATAGRID_FIELDS = ["value_punto_contatto", "timeline_tempi_scadenze"]


@adapter(IField, Interface, Interface)
@implementer(IJsonSchemaProvider)
class LeadImageJsonSchemaProvider(ObjectJsonSchemaProvider):
    def get_size_vocabulary(self):

        factory = getUtility(
            IVocabularyFactory, "design.plone.vocabularies.leadimage_dimension"
        )

        vocabulary = factory(self.context)._terms
        return {term.token: term.title for term in vocabulary if term.token}

    def get_schema(self):
        schema = super(LeadImageJsonSchemaProvider, self).get_schema()
        sizes = self.get_size_vocabulary()
        portal_type = getattr(self.request, "steps")[-1]
        portal_type_size = sizes.get(portal_type, None)
        if portal_type_size:
            msgid = _(
                "image_size_help",
                default="La dimensione dell'immagine dovrebbe essere di ${size} px",  # noqa
                mapping={"size": portal_type_size},
            )
            schema["description"] = translate(msgid, context=self.request)

        return schema


@adapter(IRow, Interface, Interface)
@implementer(IJsonSchemaProvider)
class DataGridRowJsonSchemaProvider(ObjectJsonSchemaProvider):
    def __init__(self, field, context, request):
        super().__init__(field, context, request)
        self.fieldsets = get_fieldsets(context, request, self.field.schema)

    def get_factory(self):
        return "DataGridField Row"

    def get_properties(self):
        if self.prefix:
            prefix = ".".join([self.prefix, self.field.__name__])
        else:
            prefix = self.field.__name__
        return get_jsonschema_properties(
            self.context, self.request, self.fieldsets, prefix
        )

    def additional(self):
        info = super().additional()
        properties = self.get_properties()
        required = []
        for field in iter_fields(self.fieldsets):
            name = field.field.getName()

            # Determine required fields
            if field.field.required:
                required.append(name)

            # Include field modes
            if field.mode:
                properties[name]["mode"] = field.mode

        info["fieldsets"] = [
            {
                "id": "default",
                "title": "Default",
                "fields": [x for x in properties.keys()],
            },
        ]
        info["required"] = required
        info["properties"] = properties
        return info



# -*- coding: utf-8 -*-
from .dxcontent import SerializeFolderToJson as BaseSerializer
from design.plone.contenttypes.interfaces.servizio import IServizio
from plone import api
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.serializer.converters import json_compatible
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.interface import implementer
from zope.interface import Interface


class SerializeFolderToJson(BaseSerializer):
    def __call__(self, version=None, include_items=True):
        # if include_items:
        #     self.request.form = {"fullobjects": ""}
        result = super(SerializeFolderToJson, self).__call__(
            version=version, include_items=include_items
        )

        catalog = api.portal.get_tool("portal_catalog")
        limit = 3
        query = {
            self.index: result["UID"],
            "portal_type": ["News Item"],
            "sort_on": "effective",
            "sort_order": "descending",
            "sort_limit": limit,
        }
        brains = catalog(**query)[:limit]
        result["related_news"] = [self.serialize_brain(x) for x in brains]
        return result

    def serialize_brain(self, brain):
        data = getMultiAdapter((brain, self.request), ISerializeToJsonSummary)()
        if brain.effective.Date() != "1969/12/31":
            data["effective"] = json_compatible(brain.effective)
        else:
            data["effective"] = None
        data["typology"] = brain.tipologia_notizia or ""

        return data


@implementer(ISerializeToJson)
@adapter(IServizio, Interface)
class ServizioSerializer(SerializeFolderToJson):
    index = "news_service"



# -*- coding: utf-8 -*-
from collective.taxonomy import PATH_SEPARATOR
from collective.taxonomy.interfaces import ITaxonomy
from design.plone.contenttypes.interfaces import IDesignPloneContenttypesLayer
from design.plone.contenttypes.interfaces.incarico import IIncarico
from design.plone.contenttypes.interfaces.persona import IPersona
from design.plone.contenttypes.interfaces.punto_di_contatto import IPuntoDiContatto
from plone import api
from plone.app.contenttypes.interfaces import IEvent
from plone.base.interfaces import IImageScalesAdapter
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.serializer.converters import json_compatible
from redturtle.volto.restapi.serializer.summary import (
    DefaultJSONSummarySerializer as BaseSerializer,
)
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.i18n import translate
from zope.interface import implementer
from zope.interface import Interface
from zope.schema import getFieldsInOrder

import re


RESOLVEUID_RE = re.compile(".*?/resolve[Uu]id/([^/]*)/?(.*)$")


@implementer(ISerializeToJsonSummary)
@adapter(Interface, IDesignPloneContenttypesLayer)
class DefaultJSONSummarySerializer(BaseSerializer):
    def __call__(self, force_all_metadata=False):
        res = super().__call__(force_all_metadata=force_all_metadata)
        metadata_fields = self.metadata_fields()
        if self.context.portal_type == "Persona":
            res["incarichi"] = self.get_incarichi()
        if self.context.portal_type == "Bando":
            if "bando_state" in metadata_fields or self.show_all_metadata_fields:
                res["bando_state"] = self.get_bando_state()

        if "geolocation" in metadata_fields or self.show_all_metadata_fields:
            # backward compatibility for some block templates
            if "geolocation" not in res:
                res["geolocation"] = None
                latitude = res.get("latitude", 0)
                longitude = res.get("longitude", 0)
                if latitude and longitude:
                    res["geolocation"] = {"latitude": latitude, "longitude": longitude}

        res["id"] = self.context.id

        # meta_type
        res["design_italia_meta_type"] = self.get_design_meta_type()

        # tassonomia argomenti
        if "tassonomia_argomenti" in res:
            if res["tassonomia_argomenti"]:
                res["tassonomia_argomenti"] = self.expand_tassonomia_argomenti()

        if self.is_get_call():
            res["has_children"] = self.has_children()

        return res

    def has_children(self):
        try:
            obj = self.context.getObject()
        except AttributeError:
            obj = self.context
        try:
            if obj.aq_base.keys():
                return True
        except AttributeError:
            return False
        return False

    def is_get_call(self):
        steps = self.request.steps
        if not steps:
            return False
        return steps[-1] == "GET_application_json_"

    def get_design_meta_type(self):
        ttool = api.portal.get_tool("portal_types")
        if self.context.portal_type == "News Item":
            # return translate(
            #     self.context.tipologia_notizia,
            #     domain="design.plone.contenttypes",
            #     context=self.request,
            # )
            return self.context.tipologia_notizia
        else:
            return translate(
                ttool[self.context.portal_type].Title(), context=self.request
            )

    def expand_tassonomia_argomenti(self):
        try:
            obj = self.context.getObject()
        except AttributeError:
            obj = self.context
        arguments = []
        for ref in getattr(obj, "tassonomia_argomenti", []):
            ref_obj = ref.to_object
            if not ref_obj:
                continue
            if not api.user.has_permission("View", obj=ref_obj):
                continue
            arguments.append(
                getMultiAdapter((ref_obj, self.request), ISerializeToJsonSummary)()
            )
        return arguments

    def get_bando_state(self):
        """
        È il metodo più safe per ottenere lo stato del bando
        anche se non il più veloce
        """
        bando = self.context.getObject()
        view = api.content.get_view("bando_view", context=bando, request=self.request)
        return view.getBandoState()

    # TODO: use tipo incarico from taxonomy when taxonomies are ready
    # instead of CT title
    def get_incarichi(self):
        try:
            obj = self.context.getObject()
        except AttributeError:
            obj = self.context

        return ", ".join([x.to_object.title for x in obj.incarichi_persona])


@implementer(ISerializeToJsonSummary)
@adapter(IIncarico, IDesignPloneContenttypesLayer)
class IncaricoDefaultJSONSummarySerializer(DefaultJSONSummarySerializer):
    def __call__(self, force_all_metadata=False):
        res = super().__call__(force_all_metadata=force_all_metadata)
        taxonomy = getUtility(ITaxonomy, name="collective.taxonomy.tipologia_incarico")
        taxonomy_voc = taxonomy.makeVocabulary(self.request.get("LANGUAGE"))
        if "tipologia_incarico" not in res:
            if self.context.tipologia_incarico:
                res["tipologia_incarico"] = taxonomy_voc.inv_data.get(
                    self.context.tipologia_incarico
                ).replace(PATH_SEPARATOR, "")
            else:
                res["tipologia_incarico"] = json_compatible(None)
        if "data_inizio_incarico" not in res:
            res["data_inizio_incarico"] = json_compatible(
                self.context.data_inizio_incarico
            )
        else:
            res["data_inizio_incarico"] = json_compatible(None)
        if "compensi" not in res:
            res["compensi"] = json_compatible(self.context.compensi)
        else:
            res["compensi"] = json_compatible([])
        return res


@implementer(ISerializeToJsonSummary)
@adapter(IPuntoDiContatto, IDesignPloneContenttypesLayer)
class PuntoDiContattoDefaultJSONSummarySerializer(DefaultJSONSummarySerializer):
    def __call__(self, force_all_metadata=False):
        res = super().__call__(force_all_metadata=force_all_metadata)
        res["value_punto_contatto"] = self.context.value_punto_contatto
        return res


@implementer(ISerializeToJsonSummary)
@adapter(IPersona, IDesignPloneContenttypesLayer)
class PersonaDefaultJSONSummarySerializer(DefaultJSONSummarySerializer):
    def __call__(self, force_all_metadata=False):
        res = super().__call__(force_all_metadata=force_all_metadata)
        fields = dict(getFieldsInOrder(IPersona))
        field = fields.get("foto_persona", None)
        if field:
            images_info_adapter = getMultiAdapter(
                (field, self.context, IDesignPloneContenttypesLayer)
            )
            if images_info_adapter:
                res["image_scales"] = {
                    "foto_persona": [images_info_adapter()],
                }
            res["image_field"] = "foto_persona"
        return res


@implementer(ISerializeToJsonSummary)
@adapter(IEvent, IDesignPloneContenttypesLayer)
class EventDefaultJSONSummarySerializer(DefaultJSONSummarySerializer):
    def __call__(self, force_all_metadata=False):
        res = super().__call__(force_all_metadata=force_all_metadata)

        # Il summary dell'evento riceve in ingresso un obj generico che può
        # essere un brain (gli items figli dell'evento) oppure un oggtto (il
        # parent). Gli attributi per le immagini vengono presi solo nel caso
        # del brain perché sono informazioni a catalogo. Per cui se non abbiamo
        # le informazioni, le calcoliamo come fanno gli indexer
        if not res.get("image_scales") and not res.get("image_field"):
            adapter = queryMultiAdapter(
                (self.context, self.request), IImageScalesAdapter
            )
            scales = adapter()
            if scales:
                res["image_scales"] = scales
            if "preview_image" in scales:
                res["image_field"] = "preview_image"
            elif "image" in scales:
                res["image_field"] = "image"
        return res



# -*- coding: utf-8 -*-
from design.plone.contenttypes import _
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.namedfile import field
from plone.supermodel import model
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


class IMultiFileSchema(model.Schema):
    """"""

    file_principale = field.NamedBlobFile(
        title=_("file_principale_label", default="File principale"),
        description=_(
            "file_principale_help",
            default="Inserisci il file principale di questo contenuto.",
        ),
        required=True,
    )

    formato_alternativo_1 = field.NamedBlobFile(
        title=_("formato_alternativo_1_label", default="Formato alternativo 1"),
        description=_(
            "formato_alternativo_1_help",
            default="Inserisci un eventuale formato alternativo del "
            "file principale.",
        ),
        required=False,
    )

    formato_alternativo_2 = field.NamedBlobFile(
        title=_("formato_alternativo_2_label", default="Formato alternativo 2"),
        description=_(
            "formato_alternativo_2_help",
            default="Inserisci un eventuale formato alternativo del "
            "file principale.",
        ),
        required=False,
    )


@provider(IFormFieldProvider)
class IMultiFile(IMultiFileSchema):
    """"""


@implementer(IMultiFile)
@adapter(IDexterityContent)
class MultiFile(object):
    """"""

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
from .dxcontent import SerializeFolderToJson as BaseSerializer
from design.plone.contenttypes.interfaces.bando import IBandoAgidSchema
from plone.restapi.interfaces import ISerializeToJson
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


@implementer(ISerializeToJson)
@adapter(IBandoAgidSchema, Interface)
class BandoSerializer(BaseSerializer):
    def get_approfondimenti(self, bando_view):
        """ """
        folders = bando_view.retrieveFolderDeepening()
        results = []

        for folder in folders:
            contents = bando_view.retrieveContentsOfFolderDeepening(folder["path"])
            if not contents:
                continue
            folder.update({"children": contents})
            results.append(folder)
        return results

    def __call__(self, version=None, include_items=True):
        result = super(BandoSerializer, self).__call__(
            version=version, include_items=include_items
        )
        bando_view = self.context.restrictedTraverse("bando_view")
        result["approfondimento"] = self.get_approfondimenti(bando_view)
        result["bando_state"] = bando_view.getBandoState()
        return result



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces import IDesignPloneContenttypesLayer
from plone import api
from plone.dexterity.interfaces import IDexterityContainer
from plone.dexterity.interfaces import IDexterityContent
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.serializer.dxcontent import (
    SerializeFolderToJson as BaseFolderSerializer,
)
from plone.restapi.serializer.dxcontent import SerializeToJson as BaseSerializer
from zope.component import adapter
from zope.i18n import translate
from zope.interface import implementer


@implementer(ISerializeToJson)
@adapter(IDexterityContent, IDesignPloneContenttypesLayer)
class SerializeToJson(BaseSerializer):
    def __call__(self, version=None, include_items=True):
        result = super(SerializeToJson, self).__call__(
            version=version, include_items=include_items
        )
        ttool = api.portal.get_tool("portal_types")
        if self.context.portal_type == "News Item":
            result["design_italia_meta_type"] = self.context.tipologia_notizia
        else:
            result["design_italia_meta_type"] = translate(
                ttool[self.context.portal_type].Title(), context=self.request
            )
        return result


@implementer(ISerializeToJson)
@adapter(IDexterityContainer, IDesignPloneContenttypesLayer)
class SerializeFolderToJson(BaseFolderSerializer):
    def __call__(self, version=None, include_items=True):
        result = super(SerializeFolderToJson, self).__call__(
            version=version, include_items=include_items
        )
        result["@id"] = self.context.absolute_url()
        ttool = api.portal.get_tool("portal_types")
        if self.context.portal_type == "News Item":
            try:
                tipologia_news = self.context.tipologia_notizia
            except AttributeError:
                # fallback if we don't have c.taxonomy configured yet
                tipologia_news = self.context.tipologia_notizia
            result["design_italia_meta_type"] = tipologia_news
        else:
            result["design_italia_meta_type"] = translate(
                ttool[self.context.portal_type].Title(), context=self.request
            )
        if "items_total" not in result:
            # siamo in un sotto-elemento di quello richiesto dalla query.
            #  ritorniamo il numero di elementi totale, senza doverli ritornare
            # effettivamente.
            result["items_total"] = self.context.getFolderContents().actual_result_count
        return result



# -*- coding: utf-8 -*-
from design.plone.contenttypes import _
from plone.dexterity.interfaces import IDexterityContent
from zope.globalrequest import getRequest
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class VocabItem(object):
    def __init__(self, token, value):
        self.token = token
        self.value = value


@implementer(IVocabularyFactory)
class TagsVocabulary(object):
    """ """

    def __call__(self, context):
        # Just an example list of content for our vocabulary,
        # this can be any static or dynamic data, a catalog result for example.
        items = [
            VocabItem("accesso_all_informazione", _("Accesso all'informazione")),
            VocabItem("acqua", _("Acqua")),
            VocabItem("agricoltura", _("Agricoltura")),
            VocabItem("animale_domestico", _("Animale domestico")),
            VocabItem("aria", _("Aria")),
            VocabItem("assistenza_agli_anziani", _("Assistenza agli invalidi")),
            VocabItem("assistenza_sociale", _("Assistenza sociale")),
            VocabItem("associazioni", _("Associazioni")),
            VocabItem("bilancio", _("Bilancio")),
            VocabItem("commercio_all_ingresso", _("Commercio all'ingrosso")),
            VocabItem("commercio_al_minuto", _("Commercio al minuto")),
            VocabItem("commercio_ambulante", _("Commercio ambulante")),
            VocabItem("comunicazione_istituzionale", _("Comunicazione istituzionale")),
            VocabItem("comunicazione_politica", _("Comunicazione politica")),
            VocabItem("concordi", _("Concorsi")),
            VocabItem("covid_19", _("Covid - 19")),
            VocabItem("elezioni", _("Elezioni")),
            VocabItem("energie_rinnovabili", _("Energie rinnovabili")),
            VocabItem("estero", _("Estero")),
            VocabItem("foreste", _("Foreste")),
            VocabItem("formazione_professionale", _("Formazione professionale")),
            VocabItem("gemellaggi", _("Gemellaggi")),
            VocabItem("gestione_rifiuti", _("Gestione rifiuti")),
            VocabItem("giustizia", _("Giustizia")),
            VocabItem("igiene_pubblica", _("Igiene pubblica")),
            VocabItem("immigrazione", _("Immigrazione")),
            VocabItem("imposte", _("Imposte")),
            VocabItem("imprese", _("Imprese")),
            VocabItem("inquinamento", _("Inquinamento")),
            VocabItem("integrazione_sociale", _("Integrazione sociale")),
            VocabItem("isolamento_termico", _("Isolamento termico")),
            VocabItem("istruzione", _("Istruzione")),
            VocabItem("lavoro", _("Lavoro")),
            VocabItem("matrimonio", _("Matrimonio")),
            VocabItem("mercato", _("Mercato")),
            VocabItem("mobilita_sostenibile", _("Mobilità sostenibile")),
            VocabItem("morte", _("Morte")),
            VocabItem("nascita", _("Nascita")),
            VocabItem("parcheggi", _("Parcheggi")),
            VocabItem("patrimonio_culturale", _("Patrimonio culturale")),
            VocabItem("pesca", _("Pesca")),
            VocabItem("piano_di_sviluppo", _("Piano di sviluppo")),
            VocabItem("pista_ciclabile", _("Pista ciclabile")),
            VocabItem("politica_commerciale", _("Politica commerciale")),
            VocabItem("polizia", _("Polizia")),
            VocabItem("prodotti_alimentari", _("Prodotti alimentari")),
            VocabItem("protezione_civile", _("Protezione civile")),
            VocabItem("residenza", _("Residenza")),
            VocabItem("risposta_alle_emergenze", _("Risposta alle emergenze")),
            VocabItem("sistema_giuridico", _("Sistema giuridico")),
            VocabItem("spazio_verde", _("Spazio Verde")),
            VocabItem("sport", _("Sport")),
            VocabItem("sviluppo_sostenibile", _("Sviluppo sostenibile")),
            VocabItem("tassa_sui_servizi", _("Tassa sui servizi")),
            VocabItem("tempo_libero", _("Tempo libero")),
            VocabItem("trasparenza_amministrativa", _("Trasparenza amministrativa")),
            VocabItem("trasporto_pubblico", _("Trasporto pubblico")),
            VocabItem("turismo", _("Turismo")),
            VocabItem("urbanizzazione", _("Urbanizzazione")),
            VocabItem("viaggi", _("Viaggi")),
            VocabItem("zone_pedonali", _("Zone pedonali")),
            VocabItem("ztl", _("ZTL")),
        ]

        # Fix context if you are using the vocabulary in DataGridField.
        # See https://github.com/collective/collective.z3cform.datagridfield/issues/31:  # NOQA: 501
        if not IDexterityContent.providedBy(context):
            req = getRequest()
            context = req.PARENTS[0]

        # create a list of SimpleTerm items:
        terms = []
        for item in items:
            terms.append(
                SimpleTerm(value=item.token, token=str(item.token), title=item.value)
            )
        # Create a SimpleVocabulary from the terms list and return it:
        return SimpleVocabulary(terms)


TagsVocabularyFactory = TagsVocabulary()



# -*- coding: utf-8 -*-
from collective.venue.interfaces import IVenue
from plone.restapi.behaviors import IBlocks
from plone.restapi.deserializer import json_body
from plone.restapi.deserializer.dxcontent import DeserializeFromJson
from plone.restapi.indexers import SearchableText_blocks
from plone.restapi.interfaces import IDeserializeFromJson
from zExceptions import BadRequest
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


TITLE_MAX_LEN = 160
DESCRIPTION_MAX_LEN = 160
EMPTY_BLOCK_MARKER = {"@type": "text"}
MANDATORY_RICH_TEXT_FIELDS = ["modalita_accesso"]


def new_error(message):
    return {"error": "ValidationError", "message": message}


def text_in_block(blocks):
    @implementer(IBlocks)
    class FakeObject(object):
        """
        We use a fake object to use SearchableText Indexer
        """

        def Subject(self):
            return ""

        def __init__(self, blocks, blocks_layout):
            self.blocks = blocks
            self.blocks_layout = blocks_layout
            self.id = ""
            self.title = ""
            self.description = ""

    if not blocks:
        return None

    fakeObj = FakeObject(blocks.get("blocks", ""), blocks.get("blocks_layout", ""))
    return SearchableText_blocks(fakeObj)()


@implementer(IDeserializeFromJson)
@adapter(IVenue, Interface)
class DeserializeLuogoFromJson(DeserializeFromJson):
    def __call__(
        self, validate_all=False, data=None, create=False
    ):  # noqa: ignore=C901

        if data is None:
            data = json_body(self.request)

        method = self.request.get("method")
        is_post = method == "POST"
        is_patch = method == "PATCH"
        errors = []

        title = data.get("title")
        description = data.get("description")
        geolocation = data.get("geolocation")

        if is_post:
            # Title validation
            if not title:
                errors.append(new_error("Il titolo del luogo è obbligatorio"))
            elif len(title) > TITLE_MAX_LEN:
                errors.append(
                    new_error(
                        "Il titolo può avere una lunghezza di massimo {} caratteri".format(  # noqa
                            TITLE_MAX_LEN
                        )
                    )
                )

            # description validation
            if not description:
                errors.append(new_error("La descrizione del luogo è obbligatorio"))
            elif len(description) > DESCRIPTION_MAX_LEN:
                errors.append(
                    new_error(
                        "La descrizione del luogo deve avere una lunghezza di massimo {} caratteri".format(  # noqa
                            DESCRIPTION_MAX_LEN
                        )
                    )
                )

            if (
                not geolocation
                or not geolocation.get("latitude")
                or not geolocation.get("longitude")
            ):
                errors.append(
                    new_error("La geolocalizzazione del luogo è un dato obbligatorio")
                )

            for field in MANDATORY_RICH_TEXT_FIELDS:
                if field not in data:
                    errors.append(new_error("Il campo {} è obbligatorio".format(field)))
                elif field in data and not text_in_block(data.get(field)):
                    errors.append(new_error("Il campo {} è obbligatorio".format(field)))

        if is_patch:

            # Title validation
            if "title" in data and not title:
                errors.append(new_error("Il titolo del luogo è obbligatorio"))
            if title and len(title) > TITLE_MAX_LEN:
                errors.append(
                    new_error(
                        "Il titolo può avere una lunghezza di massimo {} caratteri".format(  # noqa
                            TITLE_MAX_LEN
                        )
                    )
                )
            # description validation
            if "description" in data and not description:
                errors.append(new_error("La descrizione del luogo è obbligatorio"))
            if description and len(description) > DESCRIPTION_MAX_LEN:
                errors.append(
                    new_error(
                        "La descrizione del luogo deve avere una lunghezza di massimo {} caratteri".format(  # noqa
                            DESCRIPTION_MAX_LEN
                        )
                    )
                )

            if "geolocation" in data and (
                not geolocation
                or not geolocation.get("latitude")
                or not geolocation.get("longitude")
            ):
                errors.append(
                    new_error("La geolocalizzazione del luogo è un dato obbligatorio")
                )

            for field in MANDATORY_RICH_TEXT_FIELDS:
                if field in data and not text_in_block(data.get(field)):
                    errors.append(new_error("Il campo {} è obbligatorio".format(field)))

                # Se siamo nella patch siamo in modifica. Se siamo in modifica e siamo
                # su un sito che ha avuto upgrade alla versione pnrr può essere che dei
                # campi obbligatori #un tempo non lo fossero e quindi arriviamo fino a
                # qui
                if field not in data and not text_in_block(
                    getattr(self.context, field)
                ):
                    errors.append(new_error("Il campo {} è obbligatorio".format(field)))

            if "geolocation" not in data and not getattr(self.context, "geolocation"):
                errors.append(
                    new_error("La geolocalizzazione del luogo è un dato obbligatorio")
                )

        if errors:
            raise BadRequest(errors)
        return super(DeserializeLuogoFromJson, self).__call__(
            validate_all=False, data=data, create=False
        )



# -*- coding: utf-8 -*-
"""
We need a solution like that because for some different reasons:
 * we have to customize both SerializeToJson and SerializeFolderToJson
 * If we override SerializeToJson adding this base design_italia_meta_type
   information, we need to override SerializeFolderToJson copying the whole
   code otherwise it will use the code from original SerializeToJson due to
   inheritance
 * Using a monkey patch is the easiest way to include future changes on base
   SerializeToJson and SerializeFolderToJson classes
"""

from plone import api
from plone.restapi.batching import HypermediaBatch
from plone.restapi.deserializer import boolean_value
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.serializer.dxcontent import SerializeFolderToJson
from plone.restapi.serializer.dxcontent import SerializeToJson
from Products.CMFCore.utils import getToolByName
from zope.component import getMultiAdapter
from zope.i18n import translate


original_serialize_to_json__call__ = SerializeToJson.__call__


def design_italia_serialize_to_json_call(self, version=None, include_items=True):
    ttool = api.portal.get_tool("portal_types")
    result = original_serialize_to_json__call__(
        self, version=version, include_items=include_items
    )
    if self.context.portal_type == "News Item":
        try:
            tipologia_news = self.context.tipologia_notizia
        except AttributeError:
            # fallback if we don't have c.taxonomy configured yet
            tipologia_news = self.context.tipologia_notizia
        result["design_italia_meta_type"] = tipologia_news
    else:
        result["design_italia_meta_type"] = translate(
            ttool[self.context.portal_type].Title(), context=self.request
        )

    return result


def patch_base_serializer():
    SerializeToJson.__call__ = design_italia_serialize_to_json_call


def design_italia_serialize_folder_to_json_call(self, version=None, include_items=True):
    folder_metadata = super(SerializeFolderToJson, self).__call__(
        version=version, include_items=include_items
    )

    folder_metadata.update({"is_folderish": True})
    result = folder_metadata
    include_items = self.request.form.get("include_items", include_items)
    include_items = boolean_value(include_items)
    if include_items:
        query = self._build_query()

        catalog = getToolByName(self.context, "portal_catalog")
        brains = catalog(query)

        batch = HypermediaBatch(self.request, brains)

        # These lines generate wrong result in field @id of the returned items
        # if "fullobjects" not in self.request.form:
        #   result["@id"] = batch.canonical_url
        result["items_total"] = batch.items_total
        if batch.links:
            result["batching"] = batch.links
        if "fullobjects" in list(self.request.form):
            result["items"] = getMultiAdapter((brains, self.request), ISerializeToJson)(
                fullobjects=True
            )["items"]
        else:
            result["items"] = [
                getMultiAdapter((brain, self.request), ISerializeToJsonSummary)()
                for brain in batch
            ]
    return result


def patch_base_folder_serializer():
    SerializeFolderToJson.__call__ = design_italia_serialize_folder_to_json_call



# -*- coding: utf-8 -*-
from design.plone.contenttypes.utils import create_default_blocks
from plone import api
from Products.CMFPlone.interfaces import ISelectableConstrainTypes


def incaricoCreateHandler(incarico, event):
    """
    Complete content type incarico setup on added event, generating
    missing folders, fields, etc.

    @param incarico: Content item

    @param event: Event that triggers the method (onAdded event)
    """

    FOLDERS = [
        {"id": "compensi-file", "title": "Compensi", "contains": ("File",)},
        {
            "id": "importi-di-viaggio-e-o-servizi",
            "title": "Importi di viaggio e/o servizi",
            "contains": ("File",),
        },
    ]
    for folder in FOLDERS:
        if folder["id"] in incarico:
            continue
        suboject = api.content.create(
            type="Document", id=folder["id"], title=folder["title"], container=incarico
        )
        create_default_blocks(context=suboject)
        subobjectConstraints = ISelectableConstrainTypes(suboject)
        subobjectConstraints.setConstrainTypesMode(1)
        subobjectConstraints.setLocallyAllowedTypes(folder["contains"])


def modify_incarico(obj, event):
    """
    Se cambio il titolo dell'incarico devo verificare se l'incarico è quello che
    viene puntato dalla persona. Nel caso sia quello, devo reindicizzare il
    ruolo della persona perché potrebbe essere cambiato.
    """
    idxs = ["ruolo"]
    persona = api.relation.get(target=obj, relationship="incarichi_persona")
    if not persona:
        return
    if not persona[0].isBroken():
        persona[0].from_object.reindexObject(idxs=idxs)



# -*- coding: utf-8 -*-
"""Module where all interfaces, events and exceptions live."""
from redturtle.volto.interfaces import IRedturtleVoltoLayer
from zope.interface import Interface


class IDesignPloneContenttypesLayer(IRedturtleVoltoLayer):
    """Marker interface that defines a browser layer."""


class IDesignPloneContentType(Interface):
    """
    Marker interface for all Design Italia content-types
    """



from collective.taxonomy.interfaces import ITaxonomy
from design.plone.contenttypes import _
from plone import api
from plone.memoize import ram
from Products.Five import BrowserView
from zope.component import getUtility
from zope.interface.interfaces import ComponentLookupError

import logging
import pkg_resources
import time


JS_TEMPLATE = (
    "{portal_url}/++plone++design.plone.contenttypes/js/{name}.js?v={version}"  # noqa
)

logger = logging.getLogger(__name__)


class View(BrowserView):
    def __call__(self, *args, **kwargs):
        self.move_data()
        return super().__call__(*args, **kwargs)

    def move_data(self):
        if self.request.form.get("move", ""):
            path = self.request.form.get("to_path")
            if path:
                move_to = api.content.get(path)
                if not move_to:
                    self.context.plone_utils.addPortalMessage(
                        _("Indicated path is not valid"), "error"
                    )
                    return

                for name, value in self.request.form.items():
                    if value != "on":
                        continue

                    item = api.content.get(UID=name)

                    if item:
                        api.content.move(item, move_to)

                self.context.plone_utils.addPortalMessage(
                    _("Items moved with success"), "info"
                )
            else:
                self.context.plone_utils.addPortalMessage(
                    _("The path was not indicated"), "warning"
                )

    def news_results(self):
        news_type = self.request.form.get("news_type", "")
        path = self.request.form.get("path", "")
        query = {"portal_type": "News Item"}

        if news_type:
            query["tipologia_notizia"] = news_type
        if path:
            query["path"] = path

        return api.portal.get_tool("portal_catalog")(**query)

    def news_types(self):
        try:
            taxonomy = getUtility(
                ITaxonomy, name="collective.taxonomy.tipologia_notizia"
            )
        except ComponentLookupError:
            self.context.plone_utils.addPortalMessage(
                _("Il vocabolario dei valori non è stato trovato"), "error"
            )
            return

        return taxonomy.makeVocabulary(self.request.get("LANGUAGE"))

    @ram.cache(lambda *args: time.time() // (60 * 60))
    def get_version(self):
        return pkg_resources.get_distribution("design.plone.contenttypes").version

    def get_resource_js(self, name="move_content"):
        return JS_TEMPLATE.format(
            portal_url=api.portal.get().absolute_url(),
            name=name,
            version=self.get_version(),
        )



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces.cartella_modulistica import (
    ICartellaModulistica,
)
from plone.dexterity.utils import iterSchemata
from plone.restapi.interfaces import IExpandableElement
from plone.restapi.interfaces import IFieldSerializer
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.serializer.converters import json_compatible
from plone.restapi.services import Service
from Products.CMFCore.interfaces import IFolderish
from zope.component import adapter
from zope.component import queryMultiAdapter
from zope.interface import implementer
from zope.interface import Interface
from zope.schema import getFields


@implementer(IExpandableElement)
@adapter(ICartellaModulistica, Interface)
class ModulisticaItems(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, expand=False):
        result = {
            "modulistica-items": {
                "@id": "{}/@modulistica-items".format(self.context.absolute_url())
            }
        }
        if not expand:
            return result

        data = self.get_modulistica_data()
        if data:
            result["modulistica-items"] = {"items": data}
        return result

    def get_modulistica_data(self, context=None):
        if context is None:
            context = self.context
        res = []
        for child in context.listFolderContents():
            if child.portal_type == "Document" and child.getId() == "multimedia":
                continue

            serializer = queryMultiAdapter(
                (child, self.request), ISerializeToJsonSummary
            )
            data = serializer()
            if child.portal_type == "Document":
                for schema in iterSchemata(context):
                    for name, field in getFields(schema).items():
                        if name not in ["blocks", "blocks_layout"]:
                            continue

                        # serialize the field
                        serializer = queryMultiAdapter(
                            (field, child, self.request), IFieldSerializer
                        )
                        value = serializer()
                        data[json_compatible(name)] = value
            if IFolderish.providedBy(child):
                children = [
                    x
                    for x in self.get_modulistica_data(context=child)
                    if x.get("@type", "") not in ["Document", "CartellaModulistica"]
                ]
                if children:
                    data["items"] = children
            res.append(data)
        return res


class ModulisticaItemsGet(Service):
    def reply(self):
        data = ModulisticaItems(self.context, self.request)
        return data(expand=True)["modulistica-items"]



# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from design.plone.contenttypes import _
from plone.app.dexterity import textindexer
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class IEvento(model.Schema):
    """Marker inteerface for content type Evento"""

    sottotitolo = schema.TextLine(
        title=_("sottotitolo_label", default="Sottotitolo"),
        description=_(
            "sottotitolo_help",
            default="Indica un eventuale sottotitolo/titolo alternativo.",
        ),
        required=False,
    )

    descrizione_estesa = BlocksField(
        title=_("descrizione_estesa", default="Descrizione estesa"),
        required=True,
        description=_(
            "descrizione_estesa_help",
            default="Descrizione dettagliata e completa.",
        ),
    )

    descrizione_destinatari = BlocksField(
        title=_("descrizione_destinatari", default="Descrizione destinatari"),
        required=False,
        description=_(
            "descrizione_destinatari_help",
            default="Descrizione dei principali interlocutori dell'evento.",
        ),
    )

    orari = BlocksField(
        title=_("orari", default="Informazioni sugli orari"),
        required=False,
        description=_(
            "orari_help",
            default="Informazioni sugli orari di svolgimento dell'evento.",
        ),
    )

    prezzo = BlocksField(
        title=_("prezzo", default="Costo"),
        required=False,
        description=_(
            "prezzo_help",
            default="Eventuale costo dell'evento (se ci sono uno o più biglietti), "
            "con link all'alcquisto se disponibile",
        ),
    )

    # campi presenti nelle vecchie grafiche che abbiamo deciso di continuare a mostrare
    organizzato_da_interno = RelationList(
        title=_("organizzato_da_interno_label", default="Organizzato da"),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
        description=_(
            "organizzato_da_interno_help",
            default="Se l'evento è organizzato direttamente dal comune,"
            " indicare l'ufficio/ente organizzatore. I dati di contatto "
            "verranno presi direttamente dall'ufficio selezionato. Se l'evento"
            " non è organizzato direttamente dal comune, o si vogliono "
            "sovrascrivere alcuni dati di contatto, utilizzare i seguenti campi.",  # noqa
        ),
    )
    organizzato_da_esterno = BlocksField(
        title=_("organizzato_da_esterno_label", default="Organizzatore"),
        required=False,
        description=_(
            "organizzato_da_esterno_help",
            default="Se l'evento non è organizzato direttamente dal comune oppure ha anche un organizzatore esterno,"  # noqa
            " indicare il nome del contatto.",
        ),
    )
    supportato_da = RelationList(
        title=_("supportato_da_label", default="Evento supportato da"),
        required=False,
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        description=_(
            "supportato_da_help",
            default="Indicare gli uffici/enti che supportano l'evento.",
        ),
    )

    # campi aggiunti con il pnrr
    patrocinato_da = schema.TextLine(
        title=_("patrocinato_da_label", default="Patrocinato da"),
        required=False,
        description=_(
            "patrocinato_da_help",
            default="Indicare l'ente che supporta l'evento, se presente.",
        ),
    )

    persone_amministrazione = RelationList(
        title=_("parteciperanno_label", default="Parteciperanno (Persone)"),
        required=False,
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        description=_(
            "parteciperanno_help",
            default="Link a persone dell'amministrazione che interverranno all'evento",
        ),
    )

    a_chi_si_rivolge = BlocksField(
        title=_("a_chi_si_rivolge_label", default="A chi è rivolto"),
        required=True,
        description=_(
            "a_chi_si_rivolge_help",
            default="Descrizione testuale dei principali destinatari dell'Evento",
        ),
    )

    # custom widgets
    form.widget(
        "supportato_da",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 10,
            "selectableTypes": ["UnitaOrganizzativa"],
        },
    )
    form.widget(
        "organizzato_da_interno",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 10,
            "selectableTypes": ["Persona", "UnitaOrganizzativa", "Servizio"],
        },
    )
    form.widget(
        "persone_amministrazione",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "maximumSelectionSize": 10,
            "selectableTypes": ["Persona"],
        },
    )

    # custom fieldsets and order
    form.order_before(sottotitolo="ILeadImageBehavior.image")

    model.fieldset(
        "cose",
        label=_("cose_label", default="Cos'è"),
        fields=[
            "descrizione_estesa",
            "descrizione_destinatari",
            "persone_amministrazione",
        ],
    )
    model.fieldset(
        "date_e_orari",
        label=_("date_e_orari_label", default="Date e orari"),
        fields=["orari"],
    )
    model.fieldset("costi", label=_("costi_label", default="Costi"), fields=["prezzo"])
    model.fieldset(
        "contatti",
        label=_("contatti_label", default="Contatti"),
        fields=[
            "organizzato_da_interno",
            "organizzato_da_esterno",
            "supportato_da",
            "patrocinato_da",
        ],
    )

    textindexer.searchable("descrizione_estesa")


@implementer(IEvento)
@adapter(IDexterityContent)
class Evento(object):
    """ """

    def __init__(self, context):
        self.context = context
